# Processor to create a list of models
module Crud
  class KlassList
    def initialize
      update
    end

    def update
      # return if $all_klasses
      all_model_names = []
      Crud.model_path.each do |path_element|
        model_files = Dir[File.join(path_element, '**', '*.rb').to_s]

        # Ignore 'concerns'
        concerns_path_substring = File::SEPARATOR + 'concerns' + File::SEPARATOR
        model_files.reject! { |file_path| file_path.include?(concerns_path_substring) }

        model_names = model_files.map do |item|
          item.sub(path_element, '').sub(%r{^/}, '').sub(/\.rb$/, '').camelize.gsub(File::SEPARATOR, '::')
        end
        all_model_names += model_names
      end

      klasses = create_klasses_from_model_names(all_model_names)
      # Rails.logger.debug "create_klasses_from_model_names()"
      # klasses.each {|klass| Rails.logger.debug "#{klass}"}

      $all_klasses = remove_klasses_without_table(klasses).sort_by { |r| r.name.underscore }
      Rails.logger.debug "$all_klasses"
      $all_klasses.each { |klass| Rails.logger.debug "#{klass} (#{klass.name.underscore})" }
    end

    def all
      $all_klasses
    end

    private

    def create_klasses_from_model_names(model_names)
      output = []
      klass_names = model_names.select do |model_name|
        (model_name.constantize.is_a? Class) && !(model_name == 'ApplicationRecord')
      end
      klass_names.each do |klass_name|
        klass = Object.const_get(klass_name)
        output << klass if klass_name == klass.to_s
      rescue Exception => e
        Rails.logger.debug e.message
      end
      output
    end

    def remove_klasses_without_table(klasses)
      klasses.select do |klass|
        klass.ancestors.include?(ActiveRecord::Base) && klass.connection.table_exists?(klass.table_name)
      end
    end
  end
end

# p.to_s if p.respond_to?(:to_s) && /^<.+>$/.match(p.to_s).nil?
